@@ -519,55 +519,97 @@ static void bftw_file_free(struct bftw_cache *cache, struct bftw_file *file) {
 struct bftw_queue {
 	/** The head of the queue. */
 	struct bftw_file *head;
-	/** The tail of the queue. */
-	struct bftw_file *tail;
+	/** The insertion target. */
+	struct bftw_file **target;
 };
 
 /** Initialize a bftw_queue. */
 static void bftw_queue_init(struct bftw_queue *queue) {
 	queue->head = NULL;
-	queue->tail = NULL;
+	queue->target = &queue->head;
 }
 
-/** Add a file to the tail of the bftw_queue. */
+/** Add a file to a bftw_queue. */
 static void bftw_queue_push(struct bftw_queue *queue, struct bftw_file *file) {
 	assert(file->next == NULL);
 
-	if (!queue->head) {
-		queue->head = file;
-	}
-	if (queue->tail) {
-		queue->tail->next = file;
-	}
-	queue->tail = file;
-}
-
-/** Prepend a queue to the head of another one. */
-static void bftw_queue_prepend(struct bftw_queue *head, struct bftw_queue *tail) {
-	if (head->tail) {
-		head->tail->next = tail->head;
-	}
-	if (head->head) {
-		tail->head = head->head;
-	}
-	if (!tail->tail) {
-		tail->tail = head->tail;
-	}
-	head->head = NULL;
-	head->tail = NULL;
+	file->next = *queue->target;
+	*queue->target = file;
+	queue->target = &file->next;
 }
 
 /** Pop the next file from the head of the queue. */
 static struct bftw_file *bftw_queue_pop(struct bftw_queue *queue) {
 	struct bftw_file *file = queue->head;
 	queue->head = file->next;
-	if (queue->tail == file) {
-		queue->tail = NULL;
-	}
 	file->next = NULL;
+	if (queue->target == &file->next) {
+		queue->target = &queue->head;
+	}
 	return file;
 }
 
+/** The split phase of mergesort. */
+static struct bftw_file **bftw_sort_split(struct bftw_file **head, struct bftw_file **tail) {
+	struct bftw_file **tortoise = head, **hare = head;
+
+	while (*hare != *tail) {
+		tortoise = &(*tortoise)->next;
+		hare = &(*hare)->next;
+		if (*hare != *tail) {
+			hare = &(*hare)->next;
+		}
+	}
+
+	return tortoise;
+}
+
+/** The merge phase of mergesort. */
+static struct bftw_file **bftw_sort_merge(struct bftw_file **head, struct bftw_file **mid, struct bftw_file **tail) {
+	struct bftw_file *left = *head, *right = *mid, *end = *tail;
+	*mid = NULL;
+	*tail = NULL;
+
+	while (left || right) {
+		struct bftw_file *next;
+		if (left && (!right || strcoll(left->name, right->name) <= 0)) {
+			next = left;
+			left = left->next;
+		} else {
+			next = right;
+			right = right->next;
+		}
+
+		*head = next;
+		head = &next->next;
+	}
+
+	*head = end;
+	return head;
+}
+
+/**
+ * Sort a (sub-)list of files.
+ *
+ * @param head
+ *         The head of the (sub-)list to sort.
+ * @param tail
+ *         The tail of the (sub-)list to sort.
+ * @return
+ *         The new tail of the (sub-)list.
+ */
+static struct bftw_file **bftw_sort_files(struct bftw_file **head, struct bftw_file **tail) {
+	struct bftw_file **mid = bftw_sort_split(head, tail);
+	if (*mid == *head || *mid == *tail) {
+		return tail;
+	}
+
+	mid = bftw_sort_files(head, mid);
+	tail = bftw_sort_files(mid, tail);
+
+	return bftw_sort_merge(head, mid, tail);
+}
+
 /**
  * A directory reader.
  */
@@ -654,8 +696,8 @@ struct bftw_state {
 	struct bftw_cache cache;
 	/** The queue of directories left to explore. */
 	struct bftw_queue queue;
-	/** An intermediate queue used for depth-first searches. */
-	struct bftw_queue prequeue;
+	/** The start of the current batch of files. */
+	struct bftw_file **batch;
 
 	/** The current path. */
 	char *path;
@@ -693,7 +735,7 @@ static int bftw_state_init(struct bftw_state *state, const struct bftw_args *arg
 	}
 
 	bftw_queue_init(&state->queue);
-	bftw_queue_init(&state->prequeue);
+	state->batch = NULL;
 
 	state->path = dstralloc(0);
 	if (!state->path) {
@@ -1139,11 +1181,7 @@ static int bftw_push(struct bftw_state *state, const char *name, bool fill_id) {
 		bftw_fill_id(file, &state->ftwbuf);
 	}
 
-	if (state->strategy == BFTW_DFS) {
-		bftw_queue_push(&state->prequeue, file);
-	} else {
-		bftw_queue_push(&state->queue, file);
-	}
+	bftw_queue_push(&state->queue, file);
 
 	return 0;
 }
@@ -1187,10 +1225,6 @@ static int bftw_build_path(struct bftw_state *state) {
  * Pop the next file from the queue.
  */
 static int bftw_pop(struct bftw_state *state) {
-	if (state->strategy == BFTW_DFS) {
-		bftw_queue_prepend(&state->prequeue, &state->queue);
-	}
-
 	if (!state->queue.head) {
 		return 0;
 	}
@@ -1304,7 +1338,6 @@ static int bftw_state_destroy(struct bftw_state *state) {
 	bftw_release_reader(state, BFTW_VISIT_NONE);
 
 	bftw_release_file(state, BFTW_VISIT_NONE);
-	bftw_drain_queue(state, &state->prequeue);
 	bftw_drain_queue(state, &state->queue);
 
 	bftw_cache_destroy(&state->cache);
@@ -1314,9 +1347,9 @@ static int bftw_state_destroy(struct bftw_state *state) {
 }
 
 /**
- * Breadth-first bftw() implementation.
+ * Streaming mode: visit files as they are encountered.
  */
-static int bftw_bfs(const struct bftw_args *args) {
+static int bftw_stream(const struct bftw_args *args) {
 	struct bftw_state state;
 	if (bftw_state_init(&state, args) != 0) {
 		return -1;
@@ -1371,20 +1404,37 @@ done:
 	return bftw_state_destroy(&state);
 }
 
+/** Start a batch of files. */
+static void bftw_batch_start(struct bftw_state *state) {
+	if (state->strategy == BFTW_DFS) {
+		state->queue.target = &state->queue.head;
+	}
+	state->batch = state->queue.target;
+}
+
+/** Finish adding a batch of files. */
+static void bftw_batch_finish(struct bftw_state *state) {
+	if (state->flags & BFTW_SORT) {
+		state->queue.target = bftw_sort_files(state->batch, state->queue.target);
+	}
+}
+
 /**
- * Depth-first bftw() implementation.
+ * Batching mode: queue up all children before visiting them.
  */
-static int bftw_dfs(const struct bftw_args *args) {
+static int bftw_batch(const struct bftw_args *args) {
 	struct bftw_state state;
 	if (bftw_state_init(&state, args) != 0) {
 		return -1;
 	}
 
+	bftw_batch_start(&state);
 	for (size_t i = 0; i < args->npaths; ++i) {
 		if (bftw_push(&state, args->paths[i], false) != 0) {
 			goto done;
 		}
 	}
+	bftw_batch_finish(&state);
 
 	while (bftw_pop(&state) > 0) {
 		enum bftw_release_flags relflags = BFTW_VISIT_ALL;
@@ -1401,11 +1451,13 @@ static int bftw_dfs(const struct bftw_args *args) {
 
 		struct bftw_reader *reader = bftw_open(&state);
 
+		bftw_batch_start(&state);
 		while (bftw_reader_read(reader) > 0) {
 			if (bftw_push(&state, reader->de->d_name, false) != 0) {
 				goto done;
 			}
 		}
+		bftw_batch_finish(&state);
 
 		if (bftw_release_reader(&state, relflags) == BFTW_STOP) {
 			goto done;
@@ -1521,15 +1573,7 @@ static int bftw_ids(const struct bftw_args *args) {
 
 	while (ret == 0 && !state.quit && !state.bottom) {
 		state.bottom = true;
-
-		// bftw_bfs() is more efficient than bftw_dfs() since it visits
-		// directory entries as it reads them.  With args->strategy ==
-		// BFTW_DFS, it gives a hybrid ordering that visits immediate
-		// children first, then deeper descendants depth-first.  This
-		// doesn't matter for iterative deepening since we only visit
-		// one level at a time.
-		ret = bftw_bfs(&ids_args);
-
+		ret = args->flags & BFTW_SORT ? bftw_batch(&ids_args) : bftw_stream(&ids_args);
 		++state.depth;
 	}
 
@@ -1538,7 +1582,7 @@ static int bftw_ids(const struct bftw_args *args) {
 
 		while (ret == 0 && !state.quit && state.depth > 0) {
 			--state.depth;
-			ret = bftw_bfs(&ids_args);
+			ret = args->flags & BFTW_SORT ? bftw_batch(&ids_args) : bftw_stream(&ids_args);
 		}
 	}
 
@@ -1555,9 +1599,12 @@ static int bftw_ids(const struct bftw_args *args) {
 int bftw(const struct bftw_args *args) {
 	switch (args->strategy) {
 	case BFTW_BFS:
-		return bftw_bfs(args);
+		if (!(args->flags & BFTW_SORT)) {
+			return bftw_stream(args);
+		}
+		// Fallthrough
 	case BFTW_DFS:
-		return bftw_dfs(args);
+		return bftw_batch(args);
 	case BFTW_IDS:
 		return bftw_ids(args);
 	}
