@@ -272,6 +272,8 @@ bsd_tests=(
 
     test_f
 
+    test_s
+
     test_double_dash
     test_flag_double_dash
 
@@ -280,6 +282,9 @@ bsd_tests=(
     test_acl
     test_L_acl
 
+    test_anewer
+    test_asince
+
     test_delete
 
     test_depth_maxdepth_1
@@ -331,12 +336,16 @@ bsd_tests=(
     test_mnewer
     test_H_mnewer
 
+    test_msince
+
     test_name_slash
     test_name_slashes
 
     test_H_newer
 
     test_newerma
+    test_newermt
+    test_newermt_epoch_minus_one
 
     test_nogroup
     test_nogroup_ulimit
@@ -425,7 +434,7 @@ gnu_tests=(
     test_depth_maxdepth_2
 
     test_empty
-
+    test_empty_special
 
     test_exec_nothing
     test_exec_substring
@@ -485,6 +494,8 @@ gnu_tests=(
     test_H_newer
 
     test_newerma
+    test_newermt
+    test_newermt_epoch_minus_one
 
     test_nogroup
     test_nogroup_ulimit
@@ -520,6 +531,7 @@ gnu_tests=(
     test_printf_nul
     test_printf_Y_error
     test_printf_H
+    test_printf_u_g_ulimit
 
     test_quit
     test_quit_child
@@ -581,6 +593,12 @@ bfs_tests=(
     test_expr_flag_path
     test_expr_path_flag
 
+    # Flags
+
+    test_S_bfs
+    test_S_dfs
+    test_S_ids
+
     # Primaries
 
     test_color
@@ -663,17 +681,20 @@ sudo_tests=(
     test_xtype_bind_mount
 )
 
-if [ "$UNAME" = "Linux" ]; then
-    sudo_tests+=(
-        test_xattr
-        test_L_xattr
-    )
-else
-    bsd_tests+=(
-        test_xattr
-        test_L_xattr
-    )
-fi
+case "$UNAME" in
+    Darwin|FreeBSD)
+        bsd_tests+=(
+            test_xattr
+            test_L_xattr
+        )
+        ;;
+    *)
+        sudo_tests+=(
+            test_xattr
+            test_L_xattr
+        )
+        ;;
+esac
 
 if [ "$DEFAULT" ]; then
     POSIX=yes
@@ -1094,6 +1115,10 @@ function test_empty() {
     bfs_diff basic -empty
 }
 
+function test_empty_special() {
+    bfs_diff rainbow -empty
+}
+
 function test_gid() {
     bfs_diff basic -gid "$(id -g)"
 }
@@ -1146,6 +1171,10 @@ function test_anewer() {
     bfs_diff times -anewer times/a
 }
 
+function test_asince() {
+    bfs_diff times -asince 1991-12-14T00:01
+}
+
 function test_links() {
     bfs_diff links -type f -links 2
 }
@@ -1346,6 +1375,14 @@ function test_newerma() {
     bfs_diff times -newerma times/a
 }
 
+function test_newermt() {
+    bfs_diff times -newermt 1991-12-14T00:01
+}
+
+function test_newermt_epoch_minus_one() {
+    bfs_diff times -newermt 1969-12-31T23:59:59Z
+}
+
 function test_size() {
     bfs_diff basic -type f -size 0
 }
@@ -1716,6 +1753,16 @@ function test_f() {
     bfs_diff -f '-' -f '('
 }
 
+function test_s() {
+    invoke_bfs -s weirdnames -maxdepth 1 >"$TMP/test_s.out"
+
+    if [ "$UPDATE" ]; then
+        cp {"$TMP","$TESTS"}/test_s.out
+    else
+        diff -u {"$TESTS","$TMP"}/test_s.out
+    fi
+}
+
 function test_hidden() {
     bfs_diff weirdnames -hidden
 }
@@ -1768,6 +1815,10 @@ function test_H_mnewer() {
     bfs_diff -H times -mnewer times/l
 }
 
+function test_msince() {
+    bfs_diff times -msince 1991-12-14T00:01
+}
+
 function test_size_T() {
     bfs_diff basic -type f -size 1T
 }
@@ -1909,6 +1960,12 @@ function test_printf_H() {
     bfs_diff basic links -printf '%%p(%p) %%d(%d) %%f(%f) %%h(%h) %%H(%H) %%P(%P) %%y(%y)\n'
 }
 
+function test_printf_u_g_ulimit() {
+    closefrom 4
+    ulimit -n 16
+    [ "$(invoke_bfs deep -printf '%u %g\n' | uniq)" = "$(id -un) $(id -gn)" ]
+}
+
 function test_fstype() {
     fstype="$(invoke_bfs basic -maxdepth 0 -printf '%F\n')"
     bfs_diff basic -fstype "$fstype"
@@ -2521,7 +2578,34 @@ function test_L_xattr() {
 }
 
 function test_help() {
-    ! invoke_bfs -help | grep -E '\{...?\}'
+    invoke_bfs -help | grep -E '\{...?\}' && return 1
+    invoke_bfs -D help | grep -E '\{...?\}' && return 1
+    invoke_bfs -S help | grep -E '\{...?\}' && return 1
+    invoke_bfs -regextype help | grep -E '\{...?\}' && return 1
+
+    return 0
+}
+
+function test_S() {
+    invoke_bfs -S "$1" -s basic >"$TMP/test_S_$1.out"
+
+    if [ "$UPDATE" ]; then
+        cp {"$TMP","$TESTS"}/"test_S_$1.out"
+    else
+        diff -u {"$TESTS","$TMP"}/"test_S_$1.out"
+    fi
+}
+
+function test_S_bfs() {
+    test_S bfs
+}
+
+function test_S_dfs() {
+    test_S dfs
+}
+
+function test_S_ids() {
+    test_S ids
 }
 
 
