@@ -163,17 +163,6 @@ char *xregerror(int err, const regex_t *regex) {
 	return str;
 }
 
-int xlocaltime(const time_t *timep, struct tm *result) {
-	// Should be called before localtime_r() according to POSIX.1-2004
-	tzset();
-
-	if (localtime_r(timep, result)) {
-		return 0;
-	} else {
-		return -1;
-	}
-}
-
 void format_mode(mode_t mode, char str[11]) {
 	strcpy(str, "----------");
 
